;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
;
; NAME: newinterp.pro
;
; PURPOSE: Uses weights to compute interpolated field
;
; CATEGORY : Subroutine
;
; CALLING SEQUENCE : newinterp
;
; INPUTS :
;         ORCA_GEO grid file + ORCA_GEO weight file + Input field 
;         on ORCA grid
; OUTPUTS :
;         Output field on ORCA_GEO grid
;
; COMMON BLOCKS:
;         None
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY: 08/2002 Robinson Hordoir
;                       03/2003 Robinson Hordoir & Christian Ethe :
;                       Added duplication stripes + extrapolation on
;                       land points
;
;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------



pro newinterp


@initorca_reg
@naminterp2
@init_path2


; Get the output mask

mask_reg=read_ncdf('tmask',FILENAME=outputgrid,/NOSTRUCT,/cont_nofill)


; Get the Pointers and weights...

weight=replicate(0.,jpi,jpj,4)
pointer_i=replicate(999,jpi,jpj,4)
pointer_j=replicate(999,jpi,jpj,4)

weight(*,*,0)=read_ncdf('weight_1', FILENAME=weightfile,/NOSTRUCT,/cont_nofill)
weight(*,*,1)=read_ncdf('weight_2', FILENAME=weightfile,/NOSTRUCT,/cont_nofill)
weight(*,*,2)=read_ncdf('weight_3', FILENAME=weightfile,/NOSTRUCT,/cont_nofill)
weight(*,*,3)=read_ncdf('weight_4', FILENAME=weightfile,/NOSTRUCT,/cont_nofill)


pointer_i(*,*,0)=read_ncdf('pointer_1_i', FILENAME=weightfile,/NOSTRUCT,/cont_nofill)
pointer_i(*,*,1)=read_ncdf('pointer_2_i', FILENAME=weightfile,/NOSTRUCT,/cont_nofill)
pointer_i(*,*,2)=read_ncdf('pointer_3_i', FILENAME=weightfile,/NOSTRUCT,/cont_nofill)
pointer_i(*,*,3)=read_ncdf('pointer_4_i', FILENAME=weightfile,/NOSTRUCT,/cont_nofill)


pointer_j(*,*,0)=read_ncdf('pointer_1_j', FILENAME=weightfile,/NOSTRUCT,/cont_nofill)
pointer_j(*,*,1)=read_ncdf('pointer_2_j', FILENAME=weightfile,/NOSTRUCT,/cont_nofill)
pointer_j(*,*,2)=read_ncdf('pointer_3_j', FILENAME=weightfile,/NOSTRUCT,/cont_nofill)
pointer_j(*,*,3)=read_ncdf('pointer_4_j', FILENAME=weightfile,/NOSTRUCT,/cont_nofill)

; Get the field to be interpolated on reg geogrid

input_field=ncdf_lec(datafile,var=name_input_field)

; ici mettre : continent=where(input_field GT 1.E13)
;              input_field(continent)=0.

input_mask =ncdf_lec(inputmesh,var='tmask')

zmask=input_mask


IF keyword_set(do_rempl) then begin

; Extrapolate the input field on masked values
for jt=0,ntimesteps-1 do begin
   for jk=0, nlevel-1 do begin

    not_extrap=input_field(*,*,jk,jt)

    if (max(input_mask(*,*,jk)) GT 0 ) then input_field(*,*,jk,jt)=remplit(not_extrap,NITE=niterations,BASIQUE=input_mask(*,*,jk))

   endfor
endfor
endif


; Get the output field

output_field=replicate(0.,jpi,jpj,nlevel,ntimesteps)


for ji=0,jpi-1 do begin

   for jj=0,jpj-1 do begin
   
   
   if mask_reg(ji,jj,0) NE 0 then begin

   output_field(ji,jj,*,*)=                  $
                  weight(ji,jj,0)*input_field(pointer_i(ji,jj,0)+1,pointer_j(ji,jj,0),*,*) $
                + weight(ji,jj,1)*input_field(pointer_i(ji,jj,1)+1,pointer_j(ji,jj,1),*,*) $
                + weight(ji,jj,2)*input_field(pointer_i(ji,jj,2)+1,pointer_j(ji,jj,2),*,*) $
                + weight(ji,jj,3)*input_field(pointer_i(ji,jj,3)+1,pointer_j(ji,jj,3),*,*)
   
   endif

   endfor

endfor

; Initialisation of array for the northern boundary condition
jplus = 1
zdata_north = replicate(0.,jpi,jpj+jplus,nlevel,ntimesteps)
zdata_north(*,0:jpj-1,*,*) = output_field
zdata_inv = replicate(0.,jpi,jpj+jplus,nlevel,ntimesteps)
; Data of the input grid is duplicated for the northern boundary condition

zdata_inv(0:jpi/2 -1 ,*,*,*) = zdata_north(jpi/2:jpi-1,*,*,*) 
zdata_inv(jpi/2:jpi-1 ,*,*,*) = zdata_north(0:jpi/2-1,*,*,*) 
zdata_inv = reverse(zdata_inv,2)
zdata_north(*,jpj:jpj+jplus-1,*,*) = zdata_inv(*,jpj:jpj+jplus-1,*,*)

; Data of the input grid is duplicated for the West/east boundary condition
final_output=replicate(0.,jpiglo,jpjglo,nlevel,ntimesteps)
final_output(1:jpiglo-2,0:jpjglo-1,*,*) = zdata_north

final_output(0,*,*,*) = final_output(jpiglo-2,*,*,*)
final_output(jpiglo-1,*,*,*) = final_output(1,*,*,*)


IF keyword_set(nlower_limit) THEN BEGIN
    tab_min = WHERE( final_output LT min_value )
    IF tab_min(0) NE -1 THEN final_output(tab_min) = min_value
ENDIF

IF keyword_set(nupper_limit) THEN BEGIN
    tab_max = WHERE( final_output GT max_value )
    IF tab_max(0) NE -1 THEN final_output(tab_max) = max_value
ENDIF

final_time=ncdf_lec(datafile,var=data_time_variable)

nav_lon=ncdf_lec(outputgrid,var='nav_lon')
nav_lat=ncdf_lec(outputgrid,var='nav_lat')
nav_lev=ncdf_lec(outputgrid,var='nav_lev')

vargrid = 'T'


; Name
   idout = NCDF_CREATE(outputfile,/clobber)
   print, 'Creation du fichier Netcdf'
   NCDF_CONTROL, idout, /nofill
; Dimension

   xidout = NCDF_DIMDEF(idout, 'x', jpiglo)
   yidout = NCDF_DIMDEF(idout, 'y', jpjglo)
   didout = NCDF_DIMDEF(idout, 'z', nlevel)
   x_a    = NCDF_DIMDEF(idout, 'x_a',1)
   y_a    = NCDF_DIMDEF(idout, 'y_a',1)
   z_a    = NCDF_DIMDEF(idout, 'z_a',1)
   tidout = NCDF_DIMDEF(idout, 'time_counter', /unlimited)

; Attributes
     
   id0  = NCDF_VARDEF(idout, 'nav_lon'     , [xidout, yidout                ],/FLOAT )
   id1  = NCDF_VARDEF(idout, 'nav_lat'     , [xidout, yidout                ],/FLOAT )
   id2  = NCDF_VARDEF(idout, 'nav_lev'     , [                didout        ],/FLOAT )
   id3  = NCDF_VARDEF(idout, 'time_counter'        , [                        tidout],/FLOAT )
   id4  = NCDF_VARDEF(idout, 'time_steps'  , [                        tidout],/LONG  )
   id5  = NCDF_VARDEF(idout,name_input_field,[xidout, yidout,didout,  tidout],/DOUBLE)


; Variable 0
   NCDF_ATTPUT, idout, id0, 'units', 'degrees_east'
   NCDF_ATTPUT, idout, id0, 'long_name', 'Longitude'
   NCDF_ATTPUT, idout, id0, 'nav_model', 'Default grid'
; Variable 1
   NCDF_ATTPUT, idout, id1, 'units', 'degrees_north'
   NCDF_ATTPUT, idout, id1, 'long_name', 'Latitude'
   NCDF_ATTPUT, idout, id1, 'nav_model', 'Default grid'
; Variable 2
   NCDF_ATTPUT, idout, id2, 'units','meters'
   NCDF_ATTPUT, idout, id2, 'long_name','Depth'
   NCDF_ATTPUT, idout, id2, 'nav_model','Default grid'
; Variable 3
   NCDF_ATTPUT, idout, id3, 'units', 'seconds since 0001-01-15 12:00:00 '
   NCDF_ATTPUT, idout, id3, 'calendar','noleap'
   NCDF_ATTPUT, idout, id3, 'title', 'Time'
   NCDF_ATTPUT, idout, id3, 'long_name', 'Time axis'
   NCDF_ATTPUT, idout, id3, 'time_origin','0000-DEC-15 00:00:00'


  
   NCDF_CONTROL, idout, /ENDEF

; Writting

   NCDF_VARPUT, idout, id0 , nav_lon
   NCDF_VARPUT, idout, id1 , nav_lat
   NCDF_VARPUT, idout, id2 , nav_lev(0:nlevel-1)
   NCDF_VARPUT, idout, id3 , final_time
   NCDF_VARPUT, idout, id4 , ntimesteps
   NCDF_VARPUT, idout, id5 , final_output


   NCDF_CLOSE, idout

  
   

end








